#include "mailbox.h"

Mailbox* mailbox_new  (void);
void     mailbox_free (Mailbox* mailbox);

gboolean  mailbox_has_letter_for   (Mailbox* mailbox, const char*   recipient);
Letter*   mailbox_fetch_letter_for (Mailbox* mailbox, const char*   recipient);
void      mailbox_send_letter      (Mailbox* mailbox, const Letter* letter);

static
Letter* 
letter_clone (const Letter* letter)
{
   Letter* clone = g_new0 (Letter, 1);
   clone->recipient = g_strdup (letter->recipient);
   clone->sender    = g_strdup (letter->sender);
   clone->content   = g_strdup (letter->content);
   return clone;
}

Mailbox*
mailbox_new  (void)
{
   Mailbox* cntr = g_new0 (Mailbox, 1);
   cntr->box = NULL;
   return cntr;
}

void     
mailbox_free (Mailbox* mailbox)
{
   /* remove all letters from list, beginning with first in list */
   Letter* letter = NULL;
   
   while ((mailbox->box)!=NULL)
   {
      letter = (Letter*) g_list_nth_data (mailbox->box, 0);	 
      mailbox->box = g_list_remove (mailbox->box, letter);
      g_free (letter);
   }
   g_assert (mailbox->box == NULL);

   g_free (mailbox);
}

gboolean  
mailbox_has_letter_for   (Mailbox* mailbox, 
			const char*    recipient)
{
   Letter* letter = NULL;
   int i = 0;
   int n = g_list_length (mailbox->box);
   
   for (i=0; i<n; ++i)
   {
      letter = (Letter*) g_list_nth_data (mailbox->box, i);
      if (g_ascii_strcasecmp (letter->recipient, recipient)==0)
      {
	 return TRUE;
      }
   }
   return FALSE;
}

Letter*   
mailbox_fetch_letter_for (Mailbox* mailbox, 
			  const char*    recipient)
{
   Letter* letter = NULL;
   int i = 0;
   int n = g_list_length (mailbox->box);
   
   for (i=0; i<n; ++i)
   {
      letter = (Letter*) g_list_nth_data (mailbox->box, i);
      if (g_ascii_strcasecmp (letter->recipient, recipient)==0)
      {
	 mailbox->box = g_list_remove (mailbox->box, letter);
	 
	 return letter;
      }
   }
   return NULL;
}  

void
mailbox_send_letter     (Mailbox* mailbox, 
			 const Letter* letter)
{
   Letter* clone = letter_clone (letter);
   mailbox->box = g_list_append (mailbox->box, clone);
}


